@@ -23,6 +23,8 @@ class Corr:
 
     """
 
+    __slots__ = ["content", "N", "T", "tag", "prange"]
+
     def __init__(self, data_input, padding=[0, 0], prange=None):
         """ Initialize a Corr object.
 
@@ -732,8 +734,8 @@ class Corr:
             if len(fitrange) != 2:
                 raise Exception("fitrange has to have exactly two elements [fit_start, fit_stop]")
 
-        xs = [x for x in range(fitrange[0], fitrange[1] + 1) if not self.content[x] is None]
-        ys = [self.content[x][0] for x in range(fitrange[0], fitrange[1] + 1) if not self.content[x] is None]
+        xs = np.array([x for x in range(fitrange[0], fitrange[1] + 1) if not self.content[x] is None])
+        ys = np.array([self.content[x][0] for x in range(fitrange[0], fitrange[1] + 1) if not self.content[x] is None])
         result = least_squares(xs, ys, function, silent=silent, **kwargs)
         return result
 
@@ -786,7 +788,7 @@ class Corr:
         self.prange = prange
         return
 
-    def show(self, x_range=None, comp=None, y_range=None, logscale=False, plateau=None, fit_res=None, ylabel=None, save=None, auto_gamma=False, hide_sigma=None, references=None, title=None):
+    def show(self, x_range=None, comp=None, y_range=None, logscale=False, plateau=None, fit_res=None, fit_key=None, ylabel=None, save=None, auto_gamma=False, hide_sigma=None, references=None, title=None):
         """Plots the correlator using the tag of the correlator as label if available.
 
         Parameters
@@ -802,6 +804,8 @@ class Corr:
             Plateau value to be visualized in the figure.
         fit_res : Fit_result
             Fit_result object to be visualized.
+        fit_key : str
+            Key for the fit function in Fit_result.fit_function (for combined fits).
         ylabel : str
             Label for the y-axis.
         save : str
@@ -881,9 +885,13 @@ class Corr:
 
         if fit_res:
             x_samples = np.arange(x_range[0], x_range[1] + 1, 0.05)
-            ax1.plot(x_samples,
-                     fit_res.fit_function([o.value for o in fit_res.fit_parameters], x_samples),
-                     ls='-', marker=',', lw=2)
+            if isinstance(fit_res.fit_function, dict):
+                if fit_key:
+                    ax1.plot(x_samples, fit_res.fit_function[fit_key]([o.value for o in fit_res.fit_parameters], x_samples), ls='-', marker=',', lw=2)
+                else:
+                    raise ValueError("Please provide a 'fit_key' for visualizing combined fits.")
+            else:
+                ax1.plot(x_samples, fit_res.fit_function([o.value for o in fit_res.fit_parameters], x_samples), ls='-', marker=',', lw=2)
 
         ax1.set_xlabel(r'$x_0 / a$')
         if ylabel:
