@@ -125,8 +125,9 @@ def least_squares(x, y, func, priors=None, silent=False, **kwargs):
         It is important that all numpy functions refer to autograd.numpy, otherwise the differentiation
         will not work.
 
-    priors : list, optional
-        priors has to be a list with an entry for every parameter in the fit. The entries can either be
+    priors : dict or list, optional
+        priors can either be a dictionary with integer keys and the corresponding priors as values or
+        a list with an entry for every parameter in the fit. The entries can either be
         Obs (e.g. results from a previous fit) or strings containing a value and an error formatted like
         0.548(23), 500(40) or 0.5(0.4)
     silent : bool, optional
@@ -150,7 +151,6 @@ def least_squares(x, y, func, priors=None, silent=False, **kwargs):
         For details about how the covariance matrix is estimated see `pyerrors.obs.covariance`.
         In practice the correlation matrix is Cholesky decomposed and inverted (instead of the covariance matrix).
         This procedure should be numerically more stable as the correlation matrix is typically better conditioned (Jacobi preconditioning).
-        At the moment this option only works for `prior==None` and when no `method` is given.
     expected_chisquare : bool
         If True estimates the expected chisquare which is
         corrected by effects caused by correlated input data (default False).
@@ -166,15 +166,273 @@ def least_squares(x, y, func, priors=None, silent=False, **kwargs):
     output : Fit_result
         Parameters and information on the fitted result.
     '''
-    if priors is not None:
-        return _prior_fit(x, y, func, priors, silent=silent, **kwargs)
+    output = Fit_result()
 
-    elif (type(x) == dict and type(y) == dict and type(func) == dict):
-        return _combined_fit(x, y, func, silent=silent, **kwargs)
+    if (type(x) == dict and type(y) == dict and type(func) == dict):
+        xd = {key: anp.asarray(x[key]) for key in x}
+        yd = y
+        funcd = func
+        output.fit_function = func
     elif (type(x) == dict or type(y) == dict or type(func) == dict):
         raise TypeError("All arguments have to be dictionaries in order to perform a combined fit.")
     else:
-        return _standard_fit(x, y, func, silent=silent, **kwargs)
+        x = np.asarray(x)
+        xd = {"": x}
+        yd = {"": y}
+        funcd = {"": func}
+        output.fit_function = func
+
+    if kwargs.get('num_grad') is True:
+        jacobian = num_jacobian
+        hessian = num_hessian
+    else:
+        jacobian = auto_jacobian
+        hessian = auto_hessian
+
+    key_ls = sorted(list(xd.keys()))
+
+    if sorted(list(yd.keys())) != key_ls:
+        raise ValueError('x and y dictionaries do not contain the same keys.')
+
+    if sorted(list(funcd.keys())) != key_ls:
+        raise ValueError('x and func dictionaries do not contain the same keys.')
+
+    x_all = np.concatenate([np.array(xd[key]) for key in key_ls])
+    y_all = np.concatenate([np.array(yd[key]) for key in key_ls])
+
+    y_f = [o.value for o in y_all]
+    dy_f = [o.dvalue for o in y_all]
+
+    if len(x_all.shape) > 2:
+        raise ValueError("Unknown format for x values")
+
+    if np.any(np.asarray(dy_f) <= 0.0):
+        raise Exception("No y errors available, run the gamma method first.")
+
+    # number of fit parameters
+    n_parms_ls = []
+    for key in key_ls:
+        if not callable(funcd[key]):
+            raise TypeError('func (key=' + key + ') is not a function.')
+        if np.asarray(xd[key]).shape[-1] != len(yd[key]):
+            raise ValueError('x and y input (key=' + key + ') do not have the same length')
+        for n_loc in range(100):
+            try:
+                funcd[key](np.arange(n_loc), x_all.T[0])
+            except TypeError:
+                continue
+            except IndexError:
+                continue
+            else:
+                break
+        else:
+            raise RuntimeError("Fit function (key=" + key + ") is not valid.")
+        n_parms_ls.append(n_loc)
+
+    n_parms = max(n_parms_ls)
+    if not silent:
+        print('Fit with', n_parms, 'parameter' + 's' * (n_parms > 1))
+
+    if priors is not None:
+        if isinstance(priors, (list, np.ndarray)):
+            if n_parms != len(priors):
+                raise ValueError("'priors' does not have the correct length.")
+
+            loc_priors = []
+            for i_n, i_prior in enumerate(priors):
+                loc_priors.append(_construct_prior_obs(i_prior, i_n))
+
+            prior_mask = np.arange(len(priors))
+            output.priors = loc_priors
+
+        elif isinstance(priors, dict):
+            loc_priors = []
+            prior_mask = []
+            output.priors = {}
+            for pos, prior in priors.items():
+                if isinstance(pos, int):
+                    prior_mask.append(pos)
+                else:
+                    raise TypeError("Prior position needs to be an integer.")
+                loc_priors.append(_construct_prior_obs(prior, pos))
+
+                output.priors[pos] = loc_priors[-1]
+            if max(prior_mask) >= n_parms:
+                raise ValueError("Prior position out of range.")
+        else:
+            raise TypeError("Unkown type for `priors`.")
+
+        p_f = [o.value for o in loc_priors]
+        dp_f = [o.dvalue for o in loc_priors]
+        if np.any(np.asarray(dp_f) <= 0.0):
+            raise Exception("No prior errors available, run the gamma method first.")
+    else:
+        p_f = dp_f = np.array([])
+        prior_mask = []
+        loc_priors = []
+
+    if 'initial_guess' in kwargs:
+        x0 = kwargs.get('initial_guess')
+        if len(x0) != n_parms:
+            raise ValueError('Initial guess does not have the correct length: %d vs. %d' % (len(x0), n_parms))
+    else:
+        x0 = [0.1] * n_parms
+
+    if priors is None:
+        def general_chisqfunc_uncorr(p, ivars, pr):
+            model = anp.concatenate([anp.array(funcd[key](p, xd[key])).reshape(-1) for key in key_ls])
+            return (ivars - model) / dy_f
+    else:
+        def general_chisqfunc_uncorr(p, ivars, pr):
+            model = anp.concatenate([anp.array(funcd[key](p, xd[key])).reshape(-1) for key in key_ls])
+            return anp.concatenate(((ivars - model) / dy_f, (p[prior_mask] - pr) / dp_f))
+
+    def chisqfunc_uncorr(p):
+        return anp.sum(general_chisqfunc_uncorr(p, y_f, p_f) ** 2)
+
+    if kwargs.get('correlated_fit') is True:
+        corr = covariance(y_all, correlation=True, **kwargs)
+        covdiag = np.diag(1 / np.asarray(dy_f))
+        condn = np.linalg.cond(corr)
+        if condn > 0.1 / np.finfo(float).eps:
+            raise Exception(f"Cannot invert correlation matrix as its condition number exceeds machine precision ({condn:1.2e})")
+        if condn > 1e13:
+            warnings.warn("Correlation matrix may be ill-conditioned, condition number: {%1.2e}" % (condn), RuntimeWarning)
+        chol = np.linalg.cholesky(corr)
+        chol_inv = scipy.linalg.solve_triangular(chol, covdiag, lower=True)
+
+        def general_chisqfunc(p, ivars, pr):
+            model = anp.concatenate([anp.array(funcd[key](p, xd[key])).reshape(-1) for key in key_ls])
+            return anp.concatenate((anp.dot(chol_inv, (ivars - model)), (p[prior_mask] - pr) / dp_f))
+
+        def chisqfunc(p):
+            return anp.sum(general_chisqfunc(p, y_f, p_f) ** 2)
+    else:
+        general_chisqfunc = general_chisqfunc_uncorr
+        chisqfunc = chisqfunc_uncorr
+
+    output.method = kwargs.get('method', 'Levenberg-Marquardt')
+    if not silent:
+        print('Method:', output.method)
+
+    if output.method != 'Levenberg-Marquardt':
+        if output.method == 'migrad':
+            tolerance = 1e-4  # default value of 1e-1 set by iminuit can be problematic
+            if 'tol' in kwargs:
+                tolerance = kwargs.get('tol')
+            fit_result = iminuit.minimize(chisqfunc_uncorr, x0, tol=tolerance)  # Stopping criterion 0.002 * tol * errordef
+            if kwargs.get('correlated_fit') is True:
+                fit_result = iminuit.minimize(chisqfunc, fit_result.x, tol=tolerance)
+            output.iterations = fit_result.nfev
+        else:
+            tolerance = 1e-12
+            if 'tol' in kwargs:
+                tolerance = kwargs.get('tol')
+            fit_result = scipy.optimize.minimize(chisqfunc_uncorr, x0, method=kwargs.get('method'), tol=tolerance)
+            if kwargs.get('correlated_fit') is True:
+                fit_result = scipy.optimize.minimize(chisqfunc, fit_result.x, method=kwargs.get('method'), tol=tolerance)
+            output.iterations = fit_result.nit
+
+        chisquare = fit_result.fun
+
+    else:
+        if 'tol' in kwargs:
+            print('tol cannot be set for Levenberg-Marquardt')
+
+        def chisqfunc_residuals_uncorr(p):
+            return general_chisqfunc_uncorr(p, y_f, p_f)
+
+        fit_result = scipy.optimize.least_squares(chisqfunc_residuals_uncorr, x0, method='lm', ftol=1e-15, gtol=1e-15, xtol=1e-15)
+        if kwargs.get('correlated_fit') is True:
+
+            def chisqfunc_residuals(p):
+                return general_chisqfunc(p, y_f, p_f)
+
+            fit_result = scipy.optimize.least_squares(chisqfunc_residuals, fit_result.x, method='lm', ftol=1e-15, gtol=1e-15, xtol=1e-15)
+
+        chisquare = np.sum(fit_result.fun ** 2)
+        assert np.isclose(chisquare, chisqfunc(fit_result.x), atol=1e-14)
+
+        output.iterations = fit_result.nfev
+
+    if not fit_result.success:
+        raise Exception('The minimization procedure did not converge.')
+
+    output.chisquare = chisquare
+    output.dof = x_all.shape[-1] - n_parms + len(loc_priors)
+    output.p_value = 1 - scipy.stats.chi2.cdf(output.chisquare, output.dof)
+    if output.dof > 0:
+        output.chisquare_by_dof = output.chisquare / output.dof
+    else:
+        output.chisquare_by_dof = float('nan')
+
+    output.message = fit_result.message
+    if not silent:
+        print(fit_result.message)
+        print('chisquare/d.o.f.:', output.chisquare_by_dof)
+        print('fit parameters', fit_result.x)
+
+    def prepare_hat_matrix():
+        hat_vector = []
+        for key in key_ls:
+            if (len(xd[key]) != 0):
+                hat_vector.append(jacobian(funcd[key])(fit_result.x, xd[key]))
+        hat_vector = [item for sublist in hat_vector for item in sublist]
+        return hat_vector
+
+    if kwargs.get('expected_chisquare') is True:
+        if kwargs.get('correlated_fit') is not True:
+            W = np.diag(1 / np.asarray(dy_f))
+            cov = covariance(y_all)
+            hat_vector = prepare_hat_matrix()
+            A = W @ hat_vector
+            P_phi = A @ np.linalg.pinv(A.T @ A) @ A.T
+            expected_chisquare = np.trace((np.identity(x_all.shape[-1]) - P_phi) @ W @ cov @ W)
+            output.chisquare_by_expected_chisquare = output.chisquare / expected_chisquare
+            if not silent:
+                print('chisquare/expected_chisquare:', output.chisquare_by_expected_chisquare)
+
+    fitp = fit_result.x
+
+    try:
+        hess = hessian(chisqfunc)(fitp)
+    except TypeError:
+        raise Exception("It is required to use autograd.numpy instead of numpy within fit functions, see the documentation for details.") from None
+
+    len_y = len(y_f)
+
+    def chisqfunc_compact(d):
+        return anp.sum(general_chisqfunc(d[:n_parms], d[n_parms: n_parms + len_y], d[n_parms + len_y:]) ** 2)
+
+    jac_jac_y = hessian(chisqfunc_compact)(np.concatenate((fitp, y_f, p_f)))
+
+    # Compute hess^{-1} @ jac_jac_y[:n_parms + m, n_parms + m:] using LAPACK dgesv
+    try:
+        deriv_y = -scipy.linalg.solve(hess, jac_jac_y[:n_parms, n_parms:])
+    except np.linalg.LinAlgError:
+        raise Exception("Cannot invert hessian matrix.")
+
+    result = []
+    for i in range(n_parms):
+        result.append(derived_observable(lambda x_all, **kwargs: (x_all[0] + np.finfo(np.float64).eps) / (y_all[0].value + np.finfo(np.float64).eps) * fitp[i], list(y_all) + loc_priors, man_grad=list(deriv_y[i])))
+
+    output.fit_parameters = result
+
+    # Hotelling t-squared p-value for correlated fits.
+    if kwargs.get('correlated_fit') is True:
+        n_cov = np.min(np.vectorize(lambda x_all: x_all.N)(y_all))
+        output.t2_p_value = 1 - scipy.stats.f.cdf((n_cov - output.dof) / (output.dof * (n_cov - 1)) * output.chisquare,
+                                                  output.dof, n_cov - output.dof)
+
+    if kwargs.get('resplot') is True:
+        for key in key_ls:
+            residual_plot(xd[key], yd[key], funcd[key], result, title=key)
+
+    if kwargs.get('qqplot') is True:
+        for key in key_ls:
+            qqplot(xd[key], yd[key], funcd[key], result, title=key)
+
+    return output
 
 
 def total_least_squares(x, y, func, silent=False, **kwargs):
@@ -381,549 +639,6 @@ def total_least_squares(x, y, func, silent=False, **kwargs):
     return output
 
 
-def _prior_fit(x, y, func, priors, silent=False, **kwargs):
-    output = Fit_result()
-
-    output.fit_function = func
-
-    x = np.asarray(x)
-
-    if kwargs.get('num_grad') is True:
-        hessian = num_hessian
-    else:
-        hessian = auto_hessian
-
-    if not callable(func):
-        raise TypeError('func has to be a function.')
-
-    for i in range(100):
-        try:
-            func(np.arange(i), 0)
-        except TypeError:
-            continue
-        except IndexError:
-            continue
-        else:
-            break
-    else:
-        raise RuntimeError("Fit function is not valid.")
-
-    n_parms = i
-
-    if n_parms != len(priors):
-        raise Exception('Priors does not have the correct length.')
-
-    def extract_val_and_dval(string):
-        split_string = string.split('(')
-        if '.' in split_string[0] and '.' not in split_string[1][:-1]:
-            factor = 10 ** -len(split_string[0].partition('.')[2])
-        else:
-            factor = 1
-        return float(split_string[0]), float(split_string[1][:-1]) * factor
-
-    loc_priors = []
-    for i_n, i_prior in enumerate(priors):
-        if isinstance(i_prior, Obs):
-            loc_priors.append(i_prior)
-        else:
-            loc_val, loc_dval = extract_val_and_dval(i_prior)
-            loc_priors.append(cov_Obs(loc_val, loc_dval ** 2, '#prior' + str(i_n) + f"_{np.random.randint(2147483647):010d}"))
-
-    output.priors = loc_priors
-
-    if not silent:
-        print('Fit with', n_parms, 'parameter' + 's' * (n_parms > 1))
-
-    y_f = [o.value for o in y]
-    dy_f = [o.dvalue for o in y]
-
-    if np.any(np.asarray(dy_f) <= 0.0):
-        raise Exception('No y errors available, run the gamma method first.')
-
-    p_f = [o.value for o in loc_priors]
-    dp_f = [o.dvalue for o in loc_priors]
-
-    if np.any(np.asarray(dp_f) <= 0.0):
-        raise Exception('No prior errors available, run the gamma method first.')
-
-    if 'initial_guess' in kwargs:
-        x0 = kwargs.get('initial_guess')
-        if len(x0) != n_parms:
-            raise Exception('Initial guess does not have the correct length.')
-    else:
-        x0 = p_f
-
-    def chisqfunc(p):
-        model = func(p, x)
-        chisq = anp.sum(((y_f - model) / dy_f) ** 2) + anp.sum(((p_f - p) / dp_f) ** 2)
-        return chisq
-
-    if not silent:
-        print('Method: migrad')
-
-    m = iminuit.Minuit(chisqfunc, x0)
-    m.errordef = 1
-    m.print_level = 0
-    if 'tol' in kwargs:
-        m.tol = kwargs.get('tol')
-    else:
-        m.tol = 1e-4
-    m.migrad()
-    params = np.asarray(m.values)
-
-    output.chisquare_by_dof = m.fval / len(x)
-
-    output.method = 'migrad'
-
-    if not silent:
-        print('chisquare/d.o.f.:', output.chisquare_by_dof)
-
-    if not m.fmin.is_valid:
-        raise Exception('The minimization procedure did not converge.')
-
-    hess = hessian(chisqfunc)(params)
-    hess_inv = np.linalg.pinv(hess)
-
-    def chisqfunc_compact(d):
-        model = func(d[:n_parms], x)
-        chisq = anp.sum(((d[n_parms: n_parms + len(x)] - model) / dy_f) ** 2) + anp.sum(((d[n_parms + len(x):] - d[:n_parms]) / dp_f) ** 2)
-        return chisq
-
-    jac_jac = hessian(chisqfunc_compact)(np.concatenate((params, y_f, p_f)))
-
-    deriv = -hess_inv @ jac_jac[:n_parms, n_parms:]
-
-    result = []
-    for i in range(n_parms):
-        result.append(derived_observable(lambda x, **kwargs: (x[0] + np.finfo(np.float64).eps) / (y[0].value + np.finfo(np.float64).eps) * params[i], list(y) + list(loc_priors), man_grad=list(deriv[i])))
-
-    output.fit_parameters = result
-    output.chisquare = chisqfunc(np.asarray(params))
-
-    if kwargs.get('resplot') is True:
-        residual_plot(x, y, func, result)
-
-    if kwargs.get('qqplot') is True:
-        qqplot(x, y, func, result)
-
-    return output
-
-
-def _standard_fit(x, y, func, silent=False, **kwargs):
-    output = Fit_result()
-
-    output.fit_function = func
-
-    x = np.asarray(x)
-
-    if kwargs.get('num_grad') is True:
-        jacobian = num_jacobian
-        hessian = num_hessian
-    else:
-        jacobian = auto_jacobian
-        hessian = auto_hessian
-
-    if x.shape[-1] != len(y):
-        raise Exception('x and y input have to have the same length')
-
-    if len(x.shape) > 2:
-        raise Exception('Unknown format for x values')
-
-    if not callable(func):
-        raise TypeError('func has to be a function.')
-
-    for i in range(42):
-        try:
-            func(np.arange(i), x.T[0])
-        except TypeError:
-            continue
-        except IndexError:
-            continue
-        else:
-            break
-    else:
-        raise RuntimeError("Fit function is not valid.")
-
-    n_parms = i
-
-    if not silent:
-        print('Fit with', n_parms, 'parameter' + 's' * (n_parms > 1))
-
-    y_f = [o.value for o in y]
-    dy_f = [o.dvalue for o in y]
-
-    if np.any(np.asarray(dy_f) <= 0.0):
-        raise Exception('No y errors available, run the gamma method first.')
-
-    if 'initial_guess' in kwargs:
-        x0 = kwargs.get('initial_guess')
-        if len(x0) != n_parms:
-            raise Exception('Initial guess does not have the correct length: %d vs. %d' % (len(x0), n_parms))
-    else:
-        x0 = [0.1] * n_parms
-
-    if kwargs.get('correlated_fit') is True:
-        corr = covariance(y, correlation=True, **kwargs)
-        covdiag = np.diag(1 / np.asarray(dy_f))
-        condn = np.linalg.cond(corr)
-        if condn > 0.1 / np.finfo(float).eps:
-            raise Exception(f"Cannot invert correlation matrix as its condition number exceeds machine precision ({condn:1.2e})")
-        if condn > 1e13:
-            warnings.warn("Correlation matrix may be ill-conditioned, condition number: {%1.2e}" % (condn), RuntimeWarning)
-        chol = np.linalg.cholesky(corr)
-        chol_inv = scipy.linalg.solve_triangular(chol, covdiag, lower=True)
-
-        def chisqfunc_corr(p):
-            model = func(p, x)
-            chisq = anp.sum(anp.dot(chol_inv, (y_f - model)) ** 2)
-            return chisq
-
-    def chisqfunc(p):
-        model = func(p, x)
-        chisq = anp.sum(((y_f - model) / dy_f) ** 2)
-        return chisq
-
-    output.method = kwargs.get('method', 'Levenberg-Marquardt')
-    if not silent:
-        print('Method:', output.method)
-
-    if output.method != 'Levenberg-Marquardt':
-        if output.method == 'migrad':
-            fit_result = iminuit.minimize(chisqfunc, x0, tol=1e-4)  # Stopping criterion 0.002 * tol * errordef
-            if kwargs.get('correlated_fit') is True:
-                fit_result = iminuit.minimize(chisqfunc_corr, fit_result.x, tol=1e-4)  # Stopping criterion 0.002 * tol * errordef
-            output.iterations = fit_result.nfev
-        else:
-            fit_result = scipy.optimize.minimize(chisqfunc, x0, method=kwargs.get('method'), tol=1e-12)
-            if kwargs.get('correlated_fit') is True:
-                fit_result = scipy.optimize.minimize(chisqfunc_corr, fit_result.x, method=kwargs.get('method'), tol=1e-12)
-            output.iterations = fit_result.nit
-
-        chisquare = fit_result.fun
-
-    else:
-        if kwargs.get('correlated_fit') is True:
-            def chisqfunc_residuals_corr(p):
-                model = func(p, x)
-                chisq = anp.dot(chol_inv, (y_f - model))
-                return chisq
-
-        def chisqfunc_residuals(p):
-            model = func(p, x)
-            chisq = ((y_f - model) / dy_f)
-            return chisq
-
-        fit_result = scipy.optimize.least_squares(chisqfunc_residuals, x0, method='lm', ftol=1e-15, gtol=1e-15, xtol=1e-15)
-        if kwargs.get('correlated_fit') is True:
-            fit_result = scipy.optimize.least_squares(chisqfunc_residuals_corr, fit_result.x, method='lm', ftol=1e-15, gtol=1e-15, xtol=1e-15)
-
-        chisquare = np.sum(fit_result.fun ** 2)
-        if kwargs.get('correlated_fit') is True:
-            assert np.isclose(chisquare, chisqfunc_corr(fit_result.x), atol=1e-14)
-        else:
-            assert np.isclose(chisquare, chisqfunc(fit_result.x), atol=1e-14)
-
-        output.iterations = fit_result.nfev
-
-    if not fit_result.success:
-        raise Exception('The minimization procedure did not converge.')
-
-    if x.shape[-1] - n_parms > 0:
-        output.chisquare_by_dof = chisquare / (x.shape[-1] - n_parms)
-    else:
-        output.chisquare_by_dof = float('nan')
-
-    output.message = fit_result.message
-    if not silent:
-        print(fit_result.message)
-        print('chisquare/d.o.f.:', output.chisquare_by_dof)
-
-    if kwargs.get('expected_chisquare') is True:
-        if kwargs.get('correlated_fit') is not True:
-            W = np.diag(1 / np.asarray(dy_f))
-            cov = covariance(y)
-            A = W @ jacobian(func)(fit_result.x, x)
-            P_phi = A @ np.linalg.pinv(A.T @ A) @ A.T
-            expected_chisquare = np.trace((np.identity(x.shape[-1]) - P_phi) @ W @ cov @ W)
-            output.chisquare_by_expected_chisquare = chisquare / expected_chisquare
-            if not silent:
-                print('chisquare/expected_chisquare:',
-                      output.chisquare_by_expected_chisquare)
-
-    fitp = fit_result.x
-    try:
-        if kwargs.get('correlated_fit') is True:
-            hess = hessian(chisqfunc_corr)(fitp)
-        else:
-            hess = hessian(chisqfunc)(fitp)
-    except TypeError:
-        raise Exception("It is required to use autograd.numpy instead of numpy within fit functions, see the documentation for details.") from None
-
-    if kwargs.get('correlated_fit') is True:
-        def chisqfunc_compact(d):
-            model = func(d[:n_parms], x)
-            chisq = anp.sum(anp.dot(chol_inv, (d[n_parms:] - model)) ** 2)
-            return chisq
-
-    else:
-        def chisqfunc_compact(d):
-            model = func(d[:n_parms], x)
-            chisq = anp.sum(((d[n_parms:] - model) / dy_f) ** 2)
-            return chisq
-
-    jac_jac = hessian(chisqfunc_compact)(np.concatenate((fitp, y_f)))
-
-    # Compute hess^{-1} @ jac_jac[:n_parms, n_parms:] using LAPACK dgesv
-    try:
-        deriv = -scipy.linalg.solve(hess, jac_jac[:n_parms, n_parms:])
-    except np.linalg.LinAlgError:
-        raise Exception("Cannot invert hessian matrix.")
-
-    result = []
-    for i in range(n_parms):
-        result.append(derived_observable(lambda x, **kwargs: (x[0] + np.finfo(np.float64).eps) / (y[0].value + np.finfo(np.float64).eps) * fit_result.x[i], list(y), man_grad=list(deriv[i])))
-
-    output.fit_parameters = result
-
-    output.chisquare = chisquare
-    output.dof = x.shape[-1] - n_parms
-    output.p_value = 1 - scipy.stats.chi2.cdf(output.chisquare, output.dof)
-    # Hotelling t-squared p-value for correlated fits.
-    if kwargs.get('correlated_fit') is True:
-        n_cov = np.min(np.vectorize(lambda x: x.N)(y))
-        output.t2_p_value = 1 - scipy.stats.f.cdf((n_cov - output.dof) / (output.dof * (n_cov - 1)) * output.chisquare,
-                                                  output.dof, n_cov - output.dof)
-
-    if kwargs.get('resplot') is True:
-        residual_plot(x, y, func, result)
-
-    if kwargs.get('qqplot') is True:
-        qqplot(x, y, func, result)
-
-    return output
-
-
-def _combined_fit(x, y, func, silent=False, **kwargs):
-
-    output = Fit_result()
-    output.fit_function = func
-
-    if kwargs.get('num_grad') is True:
-        jacobian = num_jacobian
-        hessian = num_hessian
-    else:
-        jacobian = auto_jacobian
-        hessian = auto_hessian
-
-    key_ls = sorted(list(x.keys()))
-
-    if sorted(list(y.keys())) != key_ls:
-        raise Exception('x and y dictionaries do not contain the same keys.')
-
-    if sorted(list(func.keys())) != key_ls:
-        raise Exception('x and func dictionaries do not contain the same keys.')
-
-    x_all = np.concatenate([np.array(x[key]) for key in key_ls])
-    y_all = np.concatenate([np.array(y[key]) for key in key_ls])
-
-    y_f = [o.value for o in y_all]
-    dy_f = [o.dvalue for o in y_all]
-
-    if len(x_all.shape) > 2:
-        raise Exception('Unknown format for x values')
-
-    if np.any(np.asarray(dy_f) <= 0.0):
-        raise Exception('No y errors available, run the gamma method first.')
-
-    # number of fit parameters
-    n_parms_ls = []
-    for key in key_ls:
-        if not callable(func[key]):
-            raise TypeError('func (key=' + key + ') is not a function.')
-        if len(x[key]) != len(y[key]):
-            raise Exception('x and y input (key=' + key + ') do not have the same length')
-        for i in range(100):
-            try:
-                func[key](np.arange(i), x_all.T[0])
-            except TypeError:
-                continue
-            except IndexError:
-                continue
-            else:
-                break
-        else:
-            raise RuntimeError("Fit function (key=" + key + ") is not valid.")
-        n_parms = i
-        n_parms_ls.append(n_parms)
-    n_parms = max(n_parms_ls)
-    if not silent:
-        print('Fit with', n_parms, 'parameter' + 's' * (n_parms > 1))
-
-    if 'initial_guess' in kwargs:
-        x0 = kwargs.get('initial_guess')
-        if len(x0) != n_parms:
-            raise Exception('Initial guess does not have the correct length: %d vs. %d' % (len(x0), n_parms))
-    else:
-        x0 = [0.1] * n_parms
-
-    if kwargs.get('correlated_fit') is True:
-        corr = covariance(y_all, correlation=True, **kwargs)
-        covdiag = np.diag(1 / np.asarray(dy_f))
-        condn = np.linalg.cond(corr)
-        if condn > 0.1 / np.finfo(float).eps:
-            raise Exception(f"Cannot invert correlation matrix as its condition number exceeds machine precision ({condn:1.2e})")
-        if condn > 1e13:
-            warnings.warn("Correlation matrix may be ill-conditioned, condition number: {%1.2e}" % (condn), RuntimeWarning)
-        chol = np.linalg.cholesky(corr)
-        chol_inv = scipy.linalg.solve_triangular(chol, covdiag, lower=True)
-
-        def chisqfunc_corr(p):
-            model = np.concatenate([np.array(func[key](p, np.asarray(x[key]))) for key in key_ls])
-            chisq = anp.sum(anp.dot(chol_inv, (y_f - model)) ** 2)
-            return chisq
-
-    def chisqfunc(p):
-        func_list = np.concatenate([[func[k]] * len(x[k]) for k in key_ls])
-        model = anp.array([func_list[i](p, x_all[i]) for i in range(len(x_all))])
-        chisq = anp.sum(((y_f - model) / dy_f) ** 2)
-        return chisq
-
-    output.method = kwargs.get('method', 'Levenberg-Marquardt')
-    if not silent:
-        print('Method:', output.method)
-
-    if output.method != 'Levenberg-Marquardt':
-        if output.method == 'migrad':
-            tolerance = 1e-4  # default value of 1e-1 set by iminuit can be problematic
-            if 'tol' in kwargs:
-                tolerance = kwargs.get('tol')
-            fit_result = iminuit.minimize(chisqfunc, x0, tol=tolerance)  # Stopping criterion 0.002 * tol * errordef
-            if kwargs.get('correlated_fit') is True:
-                fit_result = iminuit.minimize(chisqfunc_corr, fit_result.x, tol=tolerance)
-            output.iterations = fit_result.nfev
-        else:
-            tolerance = 1e-12
-            if 'tol' in kwargs:
-                tolerance = kwargs.get('tol')
-            fit_result = scipy.optimize.minimize(chisqfunc, x0, method=kwargs.get('method'), tol=tolerance)
-            if kwargs.get('correlated_fit') is True:
-                fit_result = scipy.optimize.minimize(chisqfunc_corr, fit_result.x, method=kwargs.get('method'), tol=tolerance)
-            output.iterations = fit_result.nit
-
-        chisquare = fit_result.fun
-
-    else:
-        if kwargs.get('correlated_fit') is True:
-            def chisqfunc_residuals_corr(p):
-                model = np.concatenate([np.array(func[key](p, np.asarray(x[key]))) for key in key_ls])
-                chisq = anp.dot(chol_inv, (y_f - model))
-                return chisq
-
-        def chisqfunc_residuals(p):
-            model = np.concatenate([np.array(func[key](p, np.asarray(x[key]))) for key in key_ls])
-            chisq = ((y_f - model) / dy_f)
-            return chisq
-
-        if 'tol' in kwargs:
-            print('tol cannot be set for Levenberg-Marquardt')
-
-        fit_result = scipy.optimize.least_squares(chisqfunc_residuals, x0, method='lm', ftol=1e-15, gtol=1e-15, xtol=1e-15)
-        if kwargs.get('correlated_fit') is True:
-            fit_result = scipy.optimize.least_squares(chisqfunc_residuals_corr, fit_result.x, method='lm', ftol=1e-15, gtol=1e-15, xtol=1e-15)
-
-        chisquare = np.sum(fit_result.fun ** 2)
-        if kwargs.get('correlated_fit') is True:
-            assert np.isclose(chisquare, chisqfunc_corr(fit_result.x), atol=1e-14)
-        else:
-            assert np.isclose(chisquare, chisqfunc(fit_result.x), atol=1e-14)
-
-        output.iterations = fit_result.nfev
-
-    if not fit_result.success:
-        raise Exception('The minimization procedure did not converge.')
-
-    if x_all.shape[-1] - n_parms > 0:
-        output.chisquare = chisquare
-        output.dof = x_all.shape[-1] - n_parms
-        output.chisquare_by_dof = output.chisquare / output.dof
-        output.p_value = 1 - scipy.stats.chi2.cdf(output.chisquare, output.dof)
-    else:
-        output.chisquare_by_dof = float('nan')
-
-    output.message = fit_result.message
-    if not silent:
-        print(fit_result.message)
-        print('chisquare/d.o.f.:', output.chisquare_by_dof)
-        print('fit parameters', fit_result.x)
-
-    def prepare_hat_matrix():
-        hat_vector = []
-        for key in key_ls:
-            x_array = np.asarray(x[key])
-            if (len(x_array) != 0):
-                hat_vector.append(jacobian(func[key])(fit_result.x, x_array))
-        hat_vector = [item for sublist in hat_vector for item in sublist]
-        return hat_vector
-
-    if kwargs.get('expected_chisquare') is True:
-        if kwargs.get('correlated_fit') is not True:
-            W = np.diag(1 / np.asarray(dy_f))
-            cov = covariance(y_all)
-            hat_vector = prepare_hat_matrix()
-            A = W @ hat_vector  # hat_vector = 'jacobian(func)(fit_result.x, x)'
-            P_phi = A @ np.linalg.pinv(A.T @ A) @ A.T
-            expected_chisquare = np.trace((np.identity(x_all.shape[-1]) - P_phi) @ W @ cov @ W)
-            output.chisquare_by_expected_chisquare = output.chisquare / expected_chisquare
-            if not silent:
-                print('chisquare/expected_chisquare:', output.chisquare_by_expected_chisquare)
-
-    fitp = fit_result.x
-    if np.any(np.asarray(dy_f) <= 0.0):
-        raise Exception('No y errors available, run the gamma method first.')
-
-    try:
-        if kwargs.get('correlated_fit') is True:
-            hess = hessian(chisqfunc_corr)(fitp)
-        else:
-            hess = hessian(chisqfunc)(fitp)
-    except TypeError:
-        raise Exception("It is required to use autograd.numpy instead of numpy within fit functions, see the documentation for details.") from None
-
-    if kwargs.get('correlated_fit') is True:
-        def chisqfunc_compact(d):
-            func_list = np.concatenate([[func[k]] * len(x[k]) for k in key_ls])
-            model = anp.array([func_list[i](d[:n_parms], x_all[i]) for i in range(len(x_all))])
-            chisq = anp.sum(anp.dot(chol_inv, (d[n_parms:] - model)) ** 2)
-            return chisq
-    else:
-        def chisqfunc_compact(d):
-            func_list = np.concatenate([[func[k]] * len(x[k]) for k in key_ls])
-            model = anp.array([func_list[i](d[:n_parms], x_all[i]) for i in range(len(x_all))])
-            chisq = anp.sum(((d[n_parms:] - model) / dy_f) ** 2)
-            return chisq
-
-    jac_jac_y = hessian(chisqfunc_compact)(np.concatenate((fitp, y_f)))
-
-    # Compute hess^{-1} @ jac_jac_y[:n_parms + m, n_parms + m:] using LAPACK dgesv
-    try:
-        deriv_y = -scipy.linalg.solve(hess, jac_jac_y[:n_parms, n_parms:])
-    except np.linalg.LinAlgError:
-        raise Exception("Cannot invert hessian matrix.")
-
-    result = []
-    for i in range(n_parms):
-        result.append(derived_observable(lambda x_all, **kwargs: (x_all[0] + np.finfo(np.float64).eps) / (y_all[0].value + np.finfo(np.float64).eps) * fitp[i], list(y_all), man_grad=list(deriv_y[i])))
-
-    output.fit_parameters = result
-
-    if kwargs.get('correlated_fit') is True:
-        n_cov = np.min(np.vectorize(lambda x_all: x_all.N)(y_all))
-        output.t2_p_value = 1 - scipy.stats.f.cdf((n_cov - output.dof) / (output.dof * (n_cov - 1)) * output.chisquare,
-                                                  output.dof, n_cov - output.dof)
-
-    return output
-
-
 def fit_lin(x, y, **kwargs):
     """Performs a linear fit to y = n + m * x and returns two Obs n, m.
 
@@ -952,10 +667,10 @@ def fit_lin(x, y, **kwargs):
         out = least_squares(x, y, f, **kwargs)
         return out.fit_parameters
     else:
-        raise Exception('Unsupported types for x')
+        raise TypeError('Unsupported types for x')
 
 
-def qqplot(x, o_y, func, p):
+def qqplot(x, o_y, func, p, title=""):
     """Generates a quantile-quantile plot of the fit result which can be used to
        check if the residuals of the fit are gaussian distributed.
 
@@ -981,13 +696,15 @@ def qqplot(x, o_y, func, p):
 
     plt.xlabel('Theoretical quantiles')
     plt.ylabel('Ordered Values')
-    plt.legend()
+    plt.legend(title=title)
     plt.draw()
 
 
-def residual_plot(x, y, func, fit_res):
+def residual_plot(x, y, func, fit_res, title=""):
     """Generates a plot which compares the fit to the data and displays the corresponding residuals
 
+    For uncorrelated data the residuals are expected to be distributed ~N(0,1).
+
     Returns
     -------
     None
@@ -1005,9 +722,9 @@ def residual_plot(x, y, func, fit_res):
     ax0.set_xticklabels([])
     ax0.set_xlim([xstart, xstop])
     ax0.set_xticklabels([])
-    ax0.legend()
+    ax0.legend(title=title)
 
-    residuals = (np.asarray([o.value for o in y]) - func([o.value for o in fit_res], x)) / np.asarray([o.dvalue for o in y])
+    residuals = (np.asarray([o.value for o in y]) - func([o.value for o in fit_res], np.asarray(x))) / np.asarray([o.dvalue for o in y])
     ax1 = plt.subplot(gs[1])
     ax1.plot(x, residuals, 'ko', ls='none', markersize=5)
     ax1.tick_params(direction='out')
@@ -1086,3 +803,22 @@ def ks_test(objects=None):
     plt.draw()
 
     print(scipy.stats.kstest(p_values, 'uniform'))
+
+
+def _extract_val_and_dval(string):
+    split_string = string.split('(')
+    if '.' in split_string[0] and '.' not in split_string[1][:-1]:
+        factor = 10 ** -len(split_string[0].partition('.')[2])
+    else:
+        factor = 1
+    return float(split_string[0]), float(split_string[1][:-1]) * factor
+
+
+def _construct_prior_obs(i_prior, i_n):
+    if isinstance(i_prior, Obs):
+        return i_prior
+    elif isinstance(i_prior, str):
+        loc_val, loc_dval = _extract_val_and_dval(i_prior)
+        return cov_Obs(loc_val, loc_dval ** 2, '#prior' + str(i_n) + f"_{np.random.randint(2147483647):010d}")
+    else:
+        raise TypeError("Prior entries need to be 'Obs' or 'str'.")
