@@ -73,21 +73,21 @@ class Obs:
 
         if kwargs.get("means") is None and len(samples):
             if len(samples) != len(names):
-                raise Exception('Length of samples and names incompatible.')
+                raise ValueError('Length of samples and names incompatible.')
             if idl is not None:
                 if len(idl) != len(names):
-                    raise Exception('Length of idl incompatible with samples and names.')
+                    raise ValueError('Length of idl incompatible with samples and names.')
             name_length = len(names)
             if name_length > 1:
                 if name_length != len(set(names)):
-                    raise Exception('names are not unique.')
+                    raise ValueError('Names are not unique.')
                 if not all(isinstance(x, str) for x in names):
                     raise TypeError('All names have to be strings.')
             else:
                 if not isinstance(names[0], str):
                     raise TypeError('All names have to be strings.')
             if min(len(x) for x in samples) <= 4:
-                raise Exception('Samples have to have at least 5 entries.')
+                raise ValueError('Samples have to have at least 5 entries.')
 
         self.names = sorted(names)
         self.shape = {}
@@ -105,13 +105,13 @@ class Obs:
                 elif isinstance(idx, (list, np.ndarray)):
                     dc = np.unique(np.diff(idx))
                     if np.any(dc < 0):
-                        raise Exception("Unsorted idx for idl[%s]" % (name))
+                        raise ValueError("Unsorted idx for idl[%s]" % (name))
                     if len(dc) == 1:
                         self.idl[name] = range(idx[0], idx[-1] + dc[0], dc[0])
                     else:
                         self.idl[name] = list(idx)
                 else:
-                    raise Exception('incompatible type for idl[%s].' % (name))
+                    raise TypeError('incompatible type for idl[%s].' % (name))
         else:
             for name, sample in sorted(zip(names, samples)):
                 self.idl[name] = range(1, len(sample) + 1)
@@ -127,7 +127,7 @@ class Obs:
                 self.shape[name] = len(self.idl[name])
                 self.N += self.shape[name]
                 if len(sample) != self.shape[name]:
-                    raise Exception('Incompatible samples and idx for %s: %d vs. %d' % (name, len(sample), self.shape[name]))
+                    raise ValueError('Incompatible samples and idx for %s: %d vs. %d' % (name, len(sample), self.shape[name]))
                 self.r_values[name] = np.mean(sample)
                 self.deltas[name] = sample - self.r_values[name]
                 self._value += self.shape[name] * self.r_values[name]
@@ -289,11 +289,14 @@ class Obs:
             self.e_n_dtauint[e_name][0] = 0.0
 
             def _compute_drho(i):
-                tmp = self.e_rho[e_name][i + 1:w_max] + np.concatenate([self.e_rho[e_name][i - 1::-1], self.e_rho[e_name][1:w_max - 2 * i]]) - 2 * self.e_rho[e_name][i] * self.e_rho[e_name][1:w_max - i]
+                tmp = (self.e_rho[e_name][i + 1:w_max]
+                       + np.concatenate([self.e_rho[e_name][i - 1:None if i - w_max // 2 < 0 else 2 * (i - w_max // 2):-1],
+                                         self.e_rho[e_name][1:max(1, w_max - 2 * i)]])
+                       - 2 * self.e_rho[e_name][i] * self.e_rho[e_name][1:w_max - i])
                 self.e_drho[e_name][i] = np.sqrt(np.sum(tmp ** 2) / e_N)
 
-            _compute_drho(gapsize)
             if self.tau_exp[e_name] > 0:
+                _compute_drho(gapsize)
                 texp = self.tau_exp[e_name]
                 # Critical slowing down analysis
                 if w_max // 2 <= 1:
@@ -320,10 +323,9 @@ class Obs:
                     # Standard automatic windowing procedure
                     tau = self.S[e_name] / np.log((2 * self.e_n_tauint[e_name][gapsize::gapsize] + 1) / (2 * self.e_n_tauint[e_name][gapsize::gapsize] - 1))
                     g_w = np.exp(- np.arange(1, len(tau) + 1) / tau) - tau / np.sqrt(np.arange(1, len(tau) + 1) * e_N)
-                    for n in range(1, w_max):
-                        if n < w_max // 2 - 2:
-                            _compute_drho(gapsize * n + gapsize)
-                        if g_w[n - 1] < 0 or n >= w_max - 1:
+                    for n in range(1, w_max // gapsize):
+                        if g_w[n - 1] < 0 or n >= w_max // gapsize - 1:
+                            _compute_drho(gapsize * n)
                             n *= gapsize
                             self.e_tauint[e_name] = self.e_n_tauint[e_name][n] * (1 + (2 * n / gapsize + 1) / e_N) / (1 + 1 / e_N)  # Bias correction hep-lat/0306017 eq. (49)
                             self.e_dtauint[e_name] = self.e_n_dtauint[e_name][n]
