@@ -25,23 +25,23 @@ def test_sin2_cos2(value):
 
 
 def test_Obs_exceptions():
-    with pytest.raises(Exception):
+    with pytest.raises(ValueError):
         pe.Obs([np.random.rand(10)], ['1', '2'])
-    with pytest.raises(Exception):
+    with pytest.raises(ValueError):
         pe.Obs([np.random.rand(10)], ['1'], idl=[])
-    with pytest.raises(Exception):
+    with pytest.raises(ValueError):
         pe.Obs([np.random.rand(10), np.random.rand(10)], ['1', '1'])
-    with pytest.raises(Exception):
+    with pytest.raises(TypeError):
         pe.Obs([np.random.rand(10), np.random.rand(10)], ['1', 1])
-    with pytest.raises(Exception):
+    with pytest.raises(TypeError):
         pe.Obs([np.random.rand(10)], [1])
-    with pytest.raises(Exception):
+    with pytest.raises(ValueError):
         pe.Obs([np.random.rand(4)], ['name'])
-    with pytest.raises(Exception):
+    with pytest.raises(ValueError):
         pe.Obs([np.random.rand(5)], ['1'], idl=[[5, 3, 2 ,4 ,1]])
-    with pytest.raises(Exception):
+    with pytest.raises(TypeError):
         pe.Obs([np.random.rand(5)], ['1'], idl=['t'])
-    with pytest.raises(Exception):
+    with pytest.raises(ValueError):
         pe.Obs([np.random.rand(5)], ['1'], idl=[range(1, 8)])
 
     my_obs = pe.Obs([np.random.rand(6)], ['name'])
@@ -1085,7 +1085,7 @@ def test_non_overlapping_missing_cnfgs():
     average = (even + odd) / 2
     average.gm(S=0)
     assert np.isclose(full.value, average.value)
-    assert np.isclose(full.dvalue, average.dvalue, rtol=0.01)
+    assert np.isclose(full.dvalue, average.dvalue, rtol=0.02)
 
 
 def test_non_overlapping_operations():
@@ -1146,3 +1146,13 @@ def test_non_overlapping_operations_different_lengths():
 
         assert np.isclose(res1.value, res2.value)
         assert np.isclose(res1.dvalue, res2.dvalue, rtol=0.01)
+
+
+def test_nan_obs():
+    o = pe.pseudo_Obs(1, .1, 'test')
+    no = np.nan * o
+    no.gamma_method()
+
+    o.idl['test'] = [1, 5] + list(range(7, 2002, 2))
+    no = np.NaN * o
+    no.gamma_method()
